package info.textgrid.lab.aggregator.views;


import info.textgrid.lab.authn.RBACSession;
import info.textgrid.lab.conf.OfflineException;
import info.textgrid.lab.conf.client.ConfClient;
import info.textgrid.lab.core.model.TextGridObject;
import info.textgrid.lab.core.swtutils.AdapterUtils;

import java.text.MessageFormat;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IShowInTarget;
import org.eclipse.ui.part.ShowInContext;
import org.eclipse.ui.part.ViewPart;

import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableList.Builder;


/**
 * This sample class demonstrates how to plug-in a new
 * workbench view. The view shows data obtained from the
 * model. The sample creates a dummy model on the fly,
 * but a real implementation would connect to the model
 * available either in this or another plug-in (e.g. the workspace).
 * The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model
 * objects should be presented in the view. Each
 * view can present the same model objects using
 * different labels and icons, if needed. Alternatively,
 * a single label provider can be shared between views
 * in order to ensure that objects of the same type are
 * presented in the same way everywhere.
 * <p>
 */

public class PreviewView extends ViewPart implements IShowInTarget {

	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "info.textgrid.lab.aggregator.views.PreviewView"; //$NON-NLS-1$

	private Action action1;
	private Action action2;
	private Browser browser;

	/*
	 * The content provider class is responsible for
	 * providing objects to the view. It can wrap
	 * existing objects in adapters or simply return
	 * objects as-is. These objects may be sensitive
	 * to the current input of the view, or ignore
	 * it and always show the same content
	 * (like Task List, for example).
	 */

	/**
	 * The constructor.
	 */
	public PreviewView() {
	}

	private static String getUriString(Iterable<?> objects) {
		final Builder<String> builder = ImmutableList.<String>builder();
		for (final Object object : objects) {
			final TextGridObject tgo = AdapterUtils.getAdapter(object, TextGridObject.class);
			if (tgo != null)
				builder.add(tgo.getURI().toString());
		}
		return Joiner.on(',').join(builder.build());
	}

	private static final String MESSAGE_HTML =
			"<!DOCTYPE html>\n" + //$NON-NLS-1$
	"<html>\n" + //$NON-NLS-1$
	"<head>\n" + //$NON-NLS-1$
	"<meta name=\"viewport\" content=\"width=device-width\">\n" + //$NON-NLS-1$
	"<title>TextGridRep Preview</title>\n" + //$NON-NLS-1$
	"<style type=\"text/css\">'\n" + //$NON-NLS-1$
	"html { \n" + //$NON-NLS-1$
	"  background: #666;\n" + //$NON-NLS-1$
	"  height: 100%;\n" + //$NON-NLS-1$
	"}\n" + //$NON-NLS-1$
	"body { margin: 1cm; background: white; padding: 6pt; font-family: sans; box-shadow: 6px 6px 3px 0px rgba(0,0,0,0.75); border-radius: 3pt;}\n" + //$NON-NLS-1$
	"h1 { font-weight: normal; }\n" + //$NON-NLS-1$
	".detail { color: #444; }\n" + //$NON-NLS-1$
	"'</style>\n" + //$NON-NLS-1$
	"</head>\n" + //$NON-NLS-1$
	"<body>\n" + //$NON-NLS-1$
	"<h1>{0}</h1>\n" + //$NON-NLS-1$
	"<p>{1}</p>\n" + //$NON-NLS-1$
	"<div class=\"detail\">{2}</div>\n" + //$NON-NLS-1$
	"</body>\n" + //$NON-NLS-1$
	"</html>\n"; //$NON-NLS-1$

	private void setMessage(final String heading, final String title, final String details) {
		if (browser != null && !browser.isDisposed())
			browser.setText(MessageFormat.format(MESSAGE_HTML, heading, title, details));
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	@Override
	public void createPartControl(Composite parent) {
		browser = new Browser(parent, SWT.NONE);

		// Create the help context id for the viewer's control
		PlatformUI.getWorkbench().getHelpSystem().setHelp(browser, "info.textgrid.lab.aggregator.viewer"); //$NON-NLS-1$
		makeActions();
		hookContextMenu();
		contributeToActionBars();
		final ISelectionService selectionService = getSite().getWorkbenchWindow().getSelectionService();
		selectionService.addPostSelectionListener(new ISelectionListener() {

			@Override
			public void selectionChanged(IWorkbenchPart part,
					ISelection selection) {
				if (selection instanceof IStructuredSelection) {
                  setInputString(getUriString(((IStructuredSelection) selection).toList()));
				}
			}

		});
		setMessage(Messages.PreviewView_DefaultHeading, Messages.PreviewView_DefaultMessage,
				Messages.PreviewView_DefaultDetail);
	}

	protected void setInputString(String uriString) {
		if (browser != null && !browser.isDisposed()) {
			if (uriString == null || uriString.isEmpty()) {
                setMessage(Messages.PreviewView_DefaultHeading, Messages.PreviewView_DefaultMessage,
                                Messages.PreviewView_DefaultDetail);
			} else try {
				final String baseUrl = ConfClient.getInstance().getValue("aggregator");//$NON-NLS-1$
				final String sid = RBACSession.getInstance().getSID(false);
				browser.setUrl(baseUrl + "/html/" + uriString + "?simulate=true&sid=" + sid + "#browseHeader"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			} catch (final OfflineException e) {
				setMessage("Error contacting TextGridRep", e.getLocalizedMessage(),
						e.getCause() != null? e.getCause().getLocalizedMessage() : "");
			}

		}
	}

	private void hookContextMenu() {
//		MenuManager menuMgr = new MenuManager("#PopupMenu");
//		menuMgr.setRemoveAllWhenShown(true);
//		menuMgr.addMenuListener(new IMenuListener() {
//			public void menuAboutToShow(IMenuManager manager) {
//				PreviewView.this.fillContextMenu(manager);
//			}
//		});
//		Menu menu = menuMgr.createContextMenu(viewer.getControl());
//		viewer.getControl().setMenu(menu);
//		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		final IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
//		manager.add(action1);
//		manager.add(new Separator());
//		manager.add(action2);
	}

	private void fillContextMenu(IMenuManager manager) {
//		manager.add(action1);
//		manager.add(action2);
//		// Other plug-ins can contribute there actions here
//		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager) {
//		manager.add(action1);
//		manager.add(action2);
	}

	private void makeActions() {
//		action1 = new Action() {
//			@Override
//			public void run() {
//			}
//		};
//		action1.setText("Action 1");
//		action1.setToolTipText("Action 1 tooltip");
//		action1.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
//			getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
//
//		action2 = new Action() {
//			@Override
//			public void run() {
//				showMessage("Action 2 executed");
//			}
//		};
//		action2.setText("Action 2");
//		action2.setToolTipText("Action 2 tooltip");
//		action2.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
//				getImageDescriptor(ISharedImages.IMG_OBJS_INFO_TSK));
//		new Action() {
//			@Override
//			public void run() {
//				final ISelection selection = viewer.getSelection();
//				final Object obj = ((IStructuredSelection)selection).getFirstElement();
//				showMessage("Double-click detected on "+obj.toString());
//			}
//		};
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		browser.setFocus();
	}

	@Override
	public boolean show(ShowInContext context) {
		final ISelection selection = context.getSelection();
		if (selection != null && selection instanceof StructuredSelection) {
			setInputString(getUriString(((IStructuredSelection) selection).toList()));
			return true;
		} else if (context.getInput() != null) {
			final TextGridObject object = AdapterUtils.getAdapter(context.getInput(), TextGridObject.class);
			if (object != null) {
				setInputString(object.getURI().toString());
				return true;
			}
		}
		return false;
	}
}